/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.productionunit;

import javax.swing.JTextField;
import nl.cloudfarming.client.crop.model.CropInstance;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.lib.geotools.GeometryTools;
import org.jdesktop.beansbinding.*;
import org.jdesktop.swingx.JXDatePicker;
import org.openide.util.NbBundle;

/**
 * Cropfield Datapanel, shows cropfield properties.
 *
 * @author Gerben Feenstra
 */
@NbBundle.Messages({
    "CropFieldDataPanel.cropFieldNamePrompt=Name:",
    "CropFieldDataPanel.cropFieldNameTextField.text=Default:",
    "CropFieldDataPanel.cropFieldValidityPrompt.text=Validity:",
    "CropFieldDataPanel.cropFieldAreaPrompt.text=Area:",
    "CropFieldDataPanel.cropFieldAreaValueLabel.text= ",
    "CropFieldDataPanel.cropNamePrompt.text=Crop:",
    "CropFieldDataPanel.cropNameValueLabel.text=[Unknown]",
    "CropFieldDataPanel.classificationNamePrompt.text=Classification:",
    "CropFieldDataPanel.classificationNameValueLabel.text=[Unknown]",
    "CropFieldDataPanel.varietyNamePrompt.text=Variety:",
    "CropFieldDataPanel.varietyNameValueLabel.text=[Unknown]",})
public class CropFieldDataPanel extends javax.swing.JPanel {

    private static final String VALUE_CROP_DEFAULT_TEXT = Bundle.CropFieldDataPanel_cropNameValueLabel_text();
    private static final String VALUE_VARIETY_DEFAULT_TEXT = Bundle.CropFieldDataPanel_varietyNameValueLabel_text();
    private static final String VALUE_CLASSIFICATION_DEFAULT_TEXT = Bundle.CropFieldDataPanel_classificationNameValueLabel_text();
    private static final String PROMPT_CROPFIELD_NAME_TEXT = Bundle.CropFieldDataPanel_cropFieldNamePrompt();
    private static final String PROMPT_CROPFIELD_VALIDITY_TEXT = Bundle.CropFieldDataPanel_cropFieldValidityPrompt_text();
    private static final String PROMPT_CROPFIELD_AREA_TEXT = Bundle.CropFieldDataPanel_cropFieldAreaPrompt_text();
    private static final String PROMPT_CROP_NAME = Bundle.CropFieldDataPanel_cropNamePrompt_text();
    private static final String PROMPT_VARIETY_NAME = Bundle.CropFieldDataPanel_varietyNamePrompt_text();
    private static final String PROMPT_CLASSIFICATION_NAME = Bundle.CropFieldDataPanel_classificationNamePrompt_text();
    private final CropField cropField;
    // Binding variables
    // Name
    private BeanProperty nameBean = BeanProperty.create("text");
    private BeanProperty cropFieldNameBeanProp = BeanProperty.create(CropField.PROP_NAME);
    private Binding nameBinding;
    // Validity Start
    private BeanProperty validityStartBean = BeanProperty.create("date");
    private BeanProperty cropFieldValidityStartBeanProp = BeanProperty.create(CropField.PROP_START_DATE);
    private Binding validityStartBinding;
    //Validity End
    private BeanProperty validityEndBean = BeanProperty.create("date");
    private BeanProperty cropFieldValidityEndBeanProp = BeanProperty.create(CropField.PROP_END_DATE);
    private Binding validityEndBinding;
    private BindingGroup fieldBindingGroup = new BindingGroup();

    /**
     * Creates new form CropFieldDataPanelNew
     */
    public CropFieldDataPanel(CropField cropField) {
        assert cropField != null : "Cannot construct a CropFieldDataPanel with a null cropField";
        this.cropField = cropField;

        initComponents();
        initValues();

        // Set label texts
        cropFieldNamePrompt.setText(PROMPT_CROPFIELD_NAME_TEXT);
        cropFieldValidityPrompt.setText(PROMPT_CROPFIELD_VALIDITY_TEXT);
        cropFieldAreaPrompt.setText(PROMPT_CROPFIELD_AREA_TEXT);
        cropNamePrompt.setText(PROMPT_CROP_NAME);
        varietyNamePrompt.setText(PROMPT_VARIETY_NAME);
        bind();
    }

    /**
     * Initiate field values to values in the Field object
     */
    private void initValues() {
        cropFieldNameTextField.setText(cropField.getName());
        Double area = cropField.getGeometry() == null ? 0.0 : GeometryTools.getAreaInHA(cropField.getGeometry());
        cropFieldAreaValueLabel.setText(Double.toString(area) + " ha");
        cropFieldValidityStartDatepicker.setDate(cropField.getStartDate());
        cropFieldValidityEndDatepicker.setDate(cropField.getEndDate());

        setCropInstance(cropField.getCropInstance());
    }
    
    private void setCropInstance(CropInstance cropInstance) {
        cropNameValueLabel.setText(cropInstance.getCrop().getName());
        varietyNameValueLabel.setText(cropInstance.getVariety().getName());
        classificationNameValueLabel.setText(cropInstance.getClassification().getName());
    }

    /**
     * bind form fields to Field properties
     */
    private void bind() {
        // Name
        nameBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropFieldNameTextField, nameBean, cropField, cropFieldNameBeanProp);
        fieldBindingGroup.addBinding(nameBinding);

        // Validity Start
        validityStartBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropFieldValidityStartDatepicker, validityStartBean, cropField, cropFieldValidityStartBeanProp);
        fieldBindingGroup.addBinding(validityStartBinding);

        // Validity End
        validityEndBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropFieldValidityEndDatepicker, validityEndBean, cropField, cropFieldValidityEndBeanProp);
        fieldBindingGroup.addBinding(validityEndBinding);

        fieldBindingGroup.bind();
    }

    protected JTextField getNameField() {
        return cropFieldNameTextField;
    }

    protected JXDatePicker getValidityEndField() {
        return cropFieldValidityEndDatepicker;
    }

    protected JXDatePicker getValidityStartField() {
        return cropFieldValidityStartDatepicker;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        cropFieldNamePrompt = new javax.swing.JLabel();
        cropFieldValidityPrompt = new javax.swing.JLabel();
        cropFieldAreaPrompt = new javax.swing.JLabel();
        cropNamePrompt = new javax.swing.JLabel();
        varietyNamePrompt = new javax.swing.JLabel();
        cropFieldNameTextField = new javax.swing.JTextField();
        cropFieldAreaValueLabel = new javax.swing.JLabel();
        cropNameValueLabel = new javax.swing.JLabel();
        varietyNameValueLabel = new javax.swing.JLabel();
        cropFieldValidityStartDatepicker = new org.jdesktop.swingx.JXDatePicker();
        cropFieldValidityEndDatepicker = new org.jdesktop.swingx.JXDatePicker();
        classificationNamePrompt = new javax.swing.JLabel();
        classificationNameValueLabel = new javax.swing.JLabel();

        cropFieldNamePrompt.setText(PROMPT_CROPFIELD_NAME_TEXT);

        cropFieldValidityPrompt.setText(PROMPT_CROPFIELD_VALIDITY_TEXT);

        cropFieldAreaPrompt.setText(PROMPT_CROPFIELD_AREA_TEXT);

        cropNamePrompt.setText(PROMPT_CROP_NAME);

        varietyNamePrompt.setText(PROMPT_VARIETY_NAME);

        cropNameValueLabel.setText(VALUE_CROP_DEFAULT_TEXT);

        varietyNameValueLabel.setText(VALUE_VARIETY_DEFAULT_TEXT);

        classificationNamePrompt.setText(PROMPT_CLASSIFICATION_NAME);

        classificationNameValueLabel.setText(VALUE_VARIETY_DEFAULT_TEXT);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(varietyNamePrompt)
                        .addContainerGap(774, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(cropFieldNamePrompt)
                            .addComponent(cropFieldValidityPrompt)
                            .addComponent(cropFieldAreaPrompt)
                            .addComponent(cropNamePrompt)
                            .addComponent(classificationNamePrompt))
                        .addGap(23, 23, 23)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(cropFieldNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 127, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(cropFieldValidityStartDatepicker, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(cropFieldValidityEndDatepicker, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addContainerGap(469, Short.MAX_VALUE))
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(classificationNameValueLabel)
                                    .addComponent(cropFieldAreaValueLabel)
                                    .addComponent(cropNameValueLabel)
                                    .addComponent(varietyNameValueLabel))
                                .addGap(0, 0, Short.MAX_VALUE))))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cropFieldNamePrompt)
                    .addComponent(cropFieldNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cropFieldValidityPrompt)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(cropFieldValidityStartDatepicker, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(cropFieldValidityEndDatepicker, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cropFieldAreaValueLabel)
                    .addComponent(cropFieldAreaPrompt))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cropNameValueLabel)
                    .addComponent(cropNamePrompt))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(varietyNameValueLabel)
                    .addComponent(varietyNamePrompt))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(classificationNameValueLabel)
                    .addComponent(classificationNamePrompt))
                .addContainerGap(129, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel classificationNamePrompt;
    private javax.swing.JLabel classificationNameValueLabel;
    private javax.swing.JLabel cropFieldAreaPrompt;
    private javax.swing.JLabel cropFieldAreaValueLabel;
    private javax.swing.JLabel cropFieldNamePrompt;
    private javax.swing.JTextField cropFieldNameTextField;
    private org.jdesktop.swingx.JXDatePicker cropFieldValidityEndDatepicker;
    private javax.swing.JLabel cropFieldValidityPrompt;
    private org.jdesktop.swingx.JXDatePicker cropFieldValidityStartDatepicker;
    private javax.swing.JLabel cropNamePrompt;
    private javax.swing.JLabel cropNameValueLabel;
    private javax.swing.JLabel varietyNamePrompt;
    private javax.swing.JLabel varietyNameValueLabel;
    // End of variables declaration//GEN-END:variables
}
