/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.productionunit;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * <p>Controller for the {@link CropProductionUnitDetailsPanel} Is able to
 * validate the panel</p> <p>Will listen to changes on the
 * {@link CropProductionUnit} got via
 * {@link CropProductionUnitDetailsController#detailsPanel} {@link CropProductionUnitDetailsPanel#getCropProductionUnit()}
 * And listens to the {@link CropProductionUnitDetailsPanel#getNameField()} for
 * name changes. On change will call
 * {@link CropProductionUnitDetailsController#cropProductionUnitDetailFieldsUpdated()}</p>
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({
    "CropProductionUnitDetailsController title=Production unit details",
    "CropProductionUnitDetailsController invalid name error=Invalid name",
    "CropProductionUnitDetailsController duplicate name error=Name is not unique",
    "CropProductionUnitDetailsController start date required=Start date is required",
    "CropProductionUnitDetailsController end date required=End date is required",
    "CropProductionUnitDetailsController end date invalid=Invalid end date",
    "CropProductionUnitDetailsController crop definition required=Crop is required",})
public abstract class CropProductionUnitDetailsController<T extends CropProductionUnitDetailsPanel> {

    protected T detailsPanel;
    private ASBodyPanel component;
    private static final String ICON = "nl/cloudfarming/client/productionunit/green32.png";

    public CropProductionUnitDetailsController(T cropProductionUnitDetailsPanel) {
        this.detailsPanel = cropProductionUnitDetailsPanel;
        detailsPanel.getCropProductionUnit().addPropertyChangeListener(new CropProductionUnitPropertyChangeListener());
        //Add separate Listener for name. This is because empty names are not committed to the model object 
        detailsPanel.getNameField().getDocument().addDocumentListener(new NameChangeListener());
    }

    public ASBodyPanel getComponent() {
        if (component == null) {
            component = new ASBodyPanel(detailsPanel);
            component.setTitle(Bundle.CropProductionUnitDetailsController_title());
            component.setIcon(ImageUtilities.loadImageIcon(ICON, false));
            component.setName(Bundle.CropProductionUnitDetailsController_title());
        }
        return component;
    }

    /**
     * <p>returns if the panel is valid</p> <p> Validates if the name provided
     * is not empty</br> Validates if the start date is set</br> Validates if
     * the end date is set</br> Validates if the start date is before the end
     * date</p>
     *
     *
     * @return
     */
    public boolean isPanelValid() {
        return isNameValid() && isStartDateValid() && isEndDateValid() && isStartDateBeforeEndDate();
    }

    /**
     * <p>Validates the panel</p> <p> Validates if the name provided is not
     * empty</br> Validates if the start date is set</br> Validates if the end
     * date is set</br> Validates if the start date is before the end date</p>
     *
     * <p>If one of the checks fails sets an error on the field
     *
     * @return
     */
    public void validatePanel() {
        validateName();
        validateStartDate();
        validateEndDate();
    }

    private boolean isNameValid() {
        String userInput = detailsPanel.getNameField().getText();
        if (userInput == null || userInput.isEmpty()) {
            return false;
        }
        return true;
    }

    private boolean isStartDateValid() {
        CropProductionUnit cropProductionUnit = detailsPanel.getCropProductionUnit();
        if (cropProductionUnit.getStartDate() == null) {
            return false;
        }
        return true;
    }

    private boolean isEndDateValid() {
        CropProductionUnit cropProductionUnit = detailsPanel.getCropProductionUnit();
        if (cropProductionUnit.getEndDate() == null) {
            return false;
        }
        return true;
    }

    private boolean isStartDateBeforeEndDate() {
        CropProductionUnit cropProductionUnit = detailsPanel.getCropProductionUnit();
        if (cropProductionUnit.getEndDate() != null && cropProductionUnit.getStartDate() != null) {
            if (cropProductionUnit.getEndDate().before(cropProductionUnit.getStartDate())) {
                return false;
            }
        }
        return true;
    }

    private void validateName() {
        if (!isNameValid()) {
            detailsPanel.getNameRow().setInvalid(Bundle.CropProductionUnitDetailsController_invalid_name_error());
        } else {
            detailsPanel.getNameRow().setValid();
        }
    }

    private void validateStartDate() {
        if (!isStartDateValid()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_start_date_required());
        } else if (!isStartDateBeforeEndDate()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_end_date_invalid());
        }

    }

    private void validateEndDate() {
        if (!isEndDateValid()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_end_date_required());
        } else if (!isStartDateBeforeEndDate()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_end_date_invalid());
        }
    }

    private class CropProductionUnitPropertyChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            detailsPanel.getValidityRow().setValid();
            if (pce.getPropertyName().equals(CropProductionUnit.PROP_START_DATE)) {
                validateStartDate();
            } else if (pce.getPropertyName().equals(CropProductionUnit.PROP_END_DATE)) {
                validateEndDate();
            }

        }
    }

    private class NameChangeListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent de) {
            validateName();
        }

        @Override
        public void removeUpdate(DocumentEvent de) {
            validateName();
        }

        @Override
        public void changedUpdate(DocumentEvent de) {
            validateName();
        }
    }
}
