/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JPanel;
import javax.swing.JTextField;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.*;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.util.swing.FormRow;
import nl.cloudfarming.client.util.swing.beanbinding.NonEmptyStringValidator;
import org.jdesktop.beansbinding.*;
import org.jdesktop.swingx.JXDatePicker;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Default edit panel for {@link CropProductionUnit}
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({
    "CropProductionUnitEditPanel name label text=Name:",
    "CropProductionUnitEditPanel description label text=Description:",
    "CropProductionUnitEditPanel validity label text=Validity:",})
public class CropProductionUnitDetailsPanel extends JPanel {

    private BindingGroup cropProductionUnitBindingGroup = new BindingGroup();
    private ELProperty formPropName = ELProperty.create("${text_ON_ACTION_OR_FOCUS_LOST}");
    ;
    private BeanProperty fieldPropName = BeanProperty.create(CropProductionUnit.PROP_NAME);
    private BeanProperty formPropDescription = BeanProperty.create("text");
    private BeanProperty fieldPropDescription = BeanProperty.create(CropProductionUnit.PROP_DESCRIPTION);
    private BeanProperty formPropValidityStart = BeanProperty.create("date");
    private BeanProperty fieldPropStartDate = BeanProperty.create(CropProductionUnit.PROP_START_DATE);
    private BeanProperty formPropValidityEnd = BeanProperty.create("date");
    private BeanProperty fieldPropEndDate = BeanProperty.create(CropProductionUnit.PROP_END_DATE);
    private JTextField nameField;
    private JTextField descriptionField;
    private JXDatePicker startingDateField;
    private JXDatePicker endDateField;
    private FormRow nameRow;
    private FormRow descriptionRow;
    private FormRow validityRow;
    private final CropProductionUnit cropProductionUnit;
    protected CropInstanceSelectorVisual cropSelector;
    private static final String NAME_LABEL = Bundle.CropProductionUnitEditPanel_name_label_text();
    private static final String DESCRIPTION_LABEL = Bundle.CropProductionUnitEditPanel_description_label_text();
    private static final String VALIDITY_LABEL = Bundle.CropProductionUnitEditPanel_validity_label_text();

    /**
     * Create the panel with a {@link CropProductionUnit} all the input values
     * will be stored in the passed CropPoductionUnit
     *
     * @param cropProductionUnit
     */
    public CropProductionUnitDetailsPanel(CropProductionUnit cropProductionUnit) {
        this.cropProductionUnit = cropProductionUnit;
        initComponents();
        bind();
    }

    private void initComponents() {
        setLayout(new MigLayout("wrap"));
        nameField = new JTextField();
        nameRow = new FormRow(true, NAME_LABEL, NAME_LABEL, nameField);
        add(nameRow);
        descriptionField = new JTextField();
        descriptionRow = new FormRow(false, DESCRIPTION_LABEL, DESCRIPTION_LABEL, descriptionField);
        add(descriptionRow);
        startingDateField = new JXDatePicker();
        startingDateField.getEditor().setEditable(false);
        endDateField = new JXDatePicker();
        endDateField.getEditor().setEditable(false);
        validityRow = new FormRow(true, VALIDITY_LABEL, VALIDITY_LABEL, startingDateField, endDateField);
        add(validityRow);
        CropDefinitionManager cropDefinitionManager = Lookup.getDefault().lookup(CropDefinitionManager.class);
        cropSelector = new CropInstanceSelectorVisual(cropDefinitionManager.findAll());
        //Check if the cropInstance is set
        //If so set the selected crop
        //If not add the first selection as selected to CropProductionUnit
        if (cropProductionUnit.getCropInstance() != null) {
            cropSelector.setSelected(cropProductionUnit.getCropInstance());
        } else {
            cropProductionUnit.setCropInstance(cropSelector.getSelectedInstance());
        }
        cropSelector.addActionListener(new CropInstanceUpdater());
        add(cropSelector);
    }

    /**
     * bind form fields to CropProductionUnit properties
     */
    private void bind() {
        AutoBinding nameBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropName, nameField, formPropName);
        nameBinding.setValidator(new NonEmptyStringValidator());
        cropProductionUnitBindingGroup.addBinding(nameBinding);
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropDescription, descriptionField, formPropDescription));
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropStartDate, startingDateField, formPropValidityStart));
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropEndDate, endDateField, formPropValidityEnd));
        cropProductionUnitBindingGroup.bind();
    }

    /**
     * Updates the {@link CropProductionUnit} when a change is made in the
     * {@link CropInstanceSelectorVisual}
     */
    private class CropInstanceUpdater implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent ae) {
            cropProductionUnit.setCropInstance(cropSelector.getSelectedInstance());
        }
    }

    public CropInstanceSelectorVisual getCropSelector() {
        return cropSelector;
    }

    public JTextField getDescriptionField() {
        return descriptionField;
    }

    public JXDatePicker getEndDateField() {
        return endDateField;
    }

    public JTextField getNameField() {
        return nameField;
    }

    public JXDatePicker getStartingDateField() {
        return startingDateField;
    }

    public FormRow getDescriptionRow() {
        return descriptionRow;
    }

    public FormRow getNameRow() {
        return nameRow;
    }

    public FormRow getValidityRow() {
        return validityRow;
    }

    /**
     * Get the {@link CropProductionUnit} this panel modifies
     *
     * @return
     */
    public CropProductionUnit getCropProductionUnit() {
        return cropProductionUnit;
    }
}
