/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.ImageUtilities;
import org.openide.util.Utilities;
import org.openide.util.datatransfer.PasteType;

/**
 * Node which represents a {@link CropProductionUnit}
 *
 * @author Maciek Dulkiewicz <maciek.dulkiewicz@ordina.nl>
 */
public final class CropProductionUnitNode extends FilterNode implements Serializable {

    private static final String CROP_PRODUCTION_UNIT_ACTION_PATH = "Actions/Productionunit";
    private static final String ICON = "nl/cloudfarming/client/productionunit/green16.png";
    protected CropProductionUnit cropProductionUnit;
    protected static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.productionunit");

    public CropProductionUnitNode(Node original, CropProductionUnit cropProductionUnit) {
        super(original, Children.create(new CropProductionUnitChildFactory(cropProductionUnit), true));
        this.cropProductionUnit = cropProductionUnit;
        
        disableDelegation(DELEGATE_SET_NAME);
        disableDelegation(DELEGATE_GET_NAME);
        setName(cropProductionUnit.getName());
        cropProductionUnit.addPropertyChangeListener(CropProductionUnit.PROP_NAME, new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent pce) {
                setName((String)pce.getNewValue());
                setDisplayName(getName());
            }
        });
    }

    @Override
    public String getDisplayName() {
        return cropProductionUnit.getName();
    }

    @Override
    public Image getIcon(int type) {
        return ImageUtilities.loadImage(ICON);
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    @Override
    public boolean canCopy() {
        return true;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public boolean canDestroy() {
        return true;
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
        // copy the filternode itself instead of delegating to the the original datanode
        return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
    }

    @Override
    public Transferable clipboardCut() throws IOException {
        return clipboardCopy();
    }

    @Override
    public Transferable drag() throws IOException {
        return clipboardCopy();
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public void setName(String name) {
        super.setName(name);
        cropProductionUnit.setName(name);
        setDisplayName(name);
    }

    @Override
    public Action[] getActions(boolean context) {
        List<Action> actions = new ArrayList();
        actions.addAll(Arrays.asList(super.getActions(context)));
        for (Action action : Utilities.actionsForPath(CROP_PRODUCTION_UNIT_ACTION_PATH)) {
            actions.add(action);
        }
        return actions.toArray(new Action[actions.size()]);
    }

    @Override
    public PasteType getDropType(final Transferable transferable, int action, int index) {

        if (transferable.isDataFlavorSupported(Field.DATA_FLAVOR)) {

            return new PasteType() {

                @Override
                public Transferable paste() throws IOException {
                    try {
                        Field field = (Field) transferable.getTransferData(Field.DATA_FLAVOR);
                        cropProductionUnit.addCropFieldFromField(field);
                    } catch (UnsupportedFlavorException ex) {
                        LOGGER.log(Level.WARNING, "Need a field to convert to cropField, {0} does not contain a field.", transferable.toString());
                    }
                    return null;
                }
            };
        } else if (transferable.isDataFlavorSupported(CropField.DATA_FLAVOR)) {
            
            try {
                final CropField cropField = (CropField) transferable.getTransferData(CropField.DATA_FLAVOR);
                
                // do not drop on current parent:
                if (cropField.getParentURI().equals(cropProductionUnit.getURI())) {
                    return null;
                }
                
                // do not move to other farm:
                if (!EnterpriseIdProvider.Util.isSameFarm(cropField, cropProductionUnit)) {
                    return null;
                }
                
                // TODO: do not allow moving cropfields depending on its state (tasks already executed)
                    
                return new PasteType() {

                    @Override
                    public Transferable paste() throws IOException {
                        cropField.getParent().removeCropField(cropField);
                        cropProductionUnit.addCropField(cropField);
                        return null;
                    }
                };
                
            } catch (UnsupportedFlavorException ex) {
                LOGGER.log(Level.WARNING, "Need a cropfield to move, {0} does not contain a cropfield.", transferable.toString());
            } catch (IOException ex) {
                LOGGER.log(Level.SEVERE, "IO exception while dropping {0} {1}", new Object[]{transferable.toString(), ex});
            }
        } 
        
        return null;
    }
}
