/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.Color;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.render.GeometricalsPreviewPanel;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.api.visual.model.ObjectSceneEvent;
import org.netbeans.api.visual.model.ObjectSceneListener;
import org.netbeans.api.visual.model.ObjectState;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 * Visual panel which displays the visual properties of a
 * {@link CropProductionUnit}
 *
 * @author Merijn Zenger
 */
@MultiViewElement.Registration(displayName = "#CTL_CropProductionUnitVisualElement",
iconBase = "nl/cloudfarming/client/productionunit/green16.png",
mimeType = CropProductionUnit.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "nl-cloudfarming-client-productionunit-CropProductionUnitVisualElement",
position = 2000)
@NbBundle.Messages({
    "CTL_CropProductionUnitVisualElement=Production unit visual",
    "CropProductionUnitVisualElement border text=Crop fields",
    "CropProductionUnitVisualElement no crop productionunit warning=No preview available. There are no crop fields assigned to this production unit"
})
public class CropProductionUnitVisualElement implements MultiViewElement {

    private static final String ICON = "nl/cloudfarming/client/productionunit/green32.png";
    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private Lookup lookup;
    private CropProductionUnit cropProductionUnit;
    private ASBodyPanel bodyPanel;
    private JList cropFieldsList;
    private GeometricalsPreviewPanel cropFieldsOverviewPanel;
    private JLabel warningLabel = new JLabel(Bundle.CropProductionUnitVisualElement_no_crop_productionunit_warning());
    private static final int CROPFIELDS_OVERVIEW_PANE_WIDTH_HEIGHT = 450;
    private static final int CROPFIELDS_LIST_WIDTH = 150;
    private static final int CROPFIELDS_LIST_HEIGHT = 450;
    private static final Color BACKGROUND_COLOR = Color.WHITE;
    private CropProductionUnitPropertyChangeListener productionUnitPropertyChangeListener;

    public CropProductionUnitVisualElement(Lookup lookup) {
        this.lookup = lookup;
        this.cropProductionUnit = lookup.lookup(CropProductionUnit.class);
        productionUnitPropertyChangeListener = new CropProductionUnitPropertyChangeListener();
        cropProductionUnit.addPropertyChangeListener(productionUnitPropertyChangeListener);

    }

    /**
     * <p>Refresh this panel with a new list of {@link CropField} 's</p> Will
     * refresh the list and the preview panel
     *
     * @param cropFields
     */
    public void refreshOverview(List<CropField> cropFields) {
        refreshCropFieldPanel(cropFields);
        refreshCropFieldList(cropFields);
    }

    /**
     * <p>Refresh the crop field preview panel with a new List of
     * {@link CropField} 's</p> <p>If provided list is size 0 will set the
     * CropFieldPanel invisible and will set the Warning label visible</p>
     *
     * @param cropFields
     */
    private void refreshCropFieldPanel(List<CropField> cropFields) {
        if (cropFields.size() > 0) {
            cropFieldsOverviewPanel.setGeometricals((List<Geometrical>) (List<? extends Geometrical>) cropFields);
            cropFieldsOverviewPanel.addSelectionChangeListener(new ObjectScenenSelectionChangeListener());
            cropFieldsOverviewPanel.setVisible(true);
            warningLabel.setVisible(false);
        } else {
            cropFieldsOverviewPanel.setVisible(false);
            warningLabel.setVisible(true);
        }
    }

    /**
     * <p>Refresh the crop list with a new List of {@link CropField} 's</p>
     * <p>If provided list is size 0 will set the CropFieldList invisible and
     * will set the Warning label visible</p>
     *
     * @param cropFields
     */
    private void refreshCropFieldList(List<CropField> cropFields) {
        if (cropFields.size() > 0) {
            cropFieldsList.setListData(cropFields.toArray());
            cropFieldsList.setVisible(true);
            warningLabel.setVisible(false);
        } else {
            cropFieldsList.setVisible(false);
            warningLabel.setVisible(true);
        }
    }

    @Override
    public JComponent getVisualRepresentation() {
        if (bodyPanel == null) {
            JPanel content = new JPanel();
            content.setLayout(new MigLayout("wrap 2, insets 0 0 0 0"));
            content.setBackground(BACKGROUND_COLOR);

            content.add(warningLabel, "span");

            Palette palette = new Palette(cropProductionUnit.getColor());
            cropFieldsOverviewPanel = new GeometricalsPreviewPanel((List<Geometrical>) (List<? extends Geometrical>) cropProductionUnit.getCropFields(), palette);
            cropFieldsOverviewPanel.setBorder(new TitledBorder(Bundle.CropProductionUnitVisualElement_border_text()));
            cropFieldsOverviewPanel.setPreferredSize(new Dimension(CROPFIELDS_OVERVIEW_PANE_WIDTH_HEIGHT, CROPFIELDS_OVERVIEW_PANE_WIDTH_HEIGHT));
            cropFieldsOverviewPanel.addSelectionChangeListener(new ObjectScenenSelectionChangeListener());
            refreshCropFieldPanel(cropProductionUnit.getCropFields());
            content.add(cropFieldsOverviewPanel);

            cropFieldsList = new JList();
            cropFieldsList.addListSelectionListener(new CropFieldListListSelectionListener());
            cropFieldsList.setBorder(new TitledBorder(Bundle.CropProductionUnitVisualElement_border_text()));
            cropFieldsList.setPreferredSize(new Dimension(CROPFIELDS_LIST_WIDTH, CROPFIELDS_LIST_HEIGHT));
            refreshCropFieldList(cropProductionUnit.getCropFields());
            JPanel listPanel = new JPanel();
            listPanel.add(cropFieldsList);
            listPanel.setBackground(BACKGROUND_COLOR);
            content.add(listPanel);

            bodyPanel = new ASBodyPanel(content);
            bodyPanel.setTitle(Bundle.CTL_CropProductionUnitVisualElement());
            bodyPanel.setIcon(ImageUtilities.loadImageIcon(ICON, true));
        }
        return bodyPanel;
    }

    private void updateName() {
        if (callback != null) {
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(cropProductionUnit.getName());
            tc.setDisplayName(cropProductionUnit.getName());
            tc.setName(cropProductionUnit.getName());
            tc.setToolTipText(cropProductionUnit.getName());
        }
    }

    /**
     * Listens to list selection Changes on the JList's On change will set the
     * selected objects on the GeometricalsPreviewPanel
     */
    private class CropFieldListListSelectionListener implements ListSelectionListener {

        private boolean userSelection = true;

        @Override
        public void valueChanged(ListSelectionEvent e) {
            if (userSelection) {
                //Needed because selection works both ways and will cause stackoverflow
                userSelection = false;
                if (cropFieldsList.getSelectedValue() != null) {
                    List<CropField> selectedCropFields = cropFieldsList.getSelectedValuesList();
                    ArrayList<Geometrical> geometricals = (ArrayList<Geometrical>) (ArrayList<? extends Geometrical>) selectedCropFields;
                    ((GeometricalsPreviewPanel) cropFieldsOverviewPanel).setSelectedGeometricals(geometricals);
                }
                userSelection = true;
            }
        }
    }

    /**
     * Listens to PropertyChanges on CropProductionUnits When Crop fields are
     * changed will update Overview When the color is changed will set the
     * palette in the GeometricalsPreviewPanel
     */
    private class CropProductionUnitPropertyChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            switch (evt.getPropertyName()) {
                case CropProductionUnit.PROP_CROPFIELDS:
                    refreshOverview(cropProductionUnit.getCropFields());
                    break;
                case CropProductionUnit.PROP_COLOR:
                    cropFieldsOverviewPanel.setPalette(new Palette((Color) evt.getNewValue()));
                    break;
            }
        }
    }

    /**
     * Listener to selection changes in an ObjectScene On Change will update the
     * selection in the Crop Field list of this panel
     */
    private class ObjectScenenSelectionChangeListener implements ObjectSceneListener {

        @Override
        public void objectAdded(ObjectSceneEvent event, Object addedObject) {
        }

        @Override
        public void objectRemoved(ObjectSceneEvent event, Object removedObject) {
        }

        @Override
        public void objectStateChanged(ObjectSceneEvent event, Object changedObject, ObjectState previousState, ObjectState newState) {
        }

        @Override
        public void selectionChanged(ObjectSceneEvent event, Set<Object> previousSelection, Set<Object> newSelection) {
            int[] newSelectionInt = new int[newSelection.size()];
            int index = 0;
            for (int i = 0; i < cropFieldsList.getModel().getSize(); i++) {
                CropField cropField = (CropField) cropFieldsList.getModel().getElementAt(i);
                if (newSelection.contains(cropField)) {
                    newSelectionInt[index] = i;
                    index++;
                }
            }
            cropFieldsList.setSelectedIndices(newSelectionInt);
        }

        @Override
        public void highlightingChanged(ObjectSceneEvent event, Set<Object> previousHighlighting, Set<Object> newHighlighting) {
        }

        @Override
        public void hoverChanged(ObjectSceneEvent event, Object previousHoveredObject, Object newHoveredObject) {
        }

        @Override
        public void focusChanged(ObjectSceneEvent event, Object previousFocusedObject, Object newFocusedObject) {
        }
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    /**
     * Get the actions for this element. Has no actions
     *
     * @return
     */
    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return lookup;
    }

    @Override
    public void componentOpened() {
        if (cropProductionUnit != null) {
            cropProductionUnit.addPropertyChangeListener(productionUnitPropertyChangeListener);
        }
    }

    @Override
    public void componentClosed() {
        if (cropProductionUnit != null) {
            cropProductionUnit.removePropertyChangeListener(productionUnitPropertyChangeListener);
        }
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback mvec) {
        this.callback = mvec;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }
}
